'use strict';

const TAG = "YunFunction:: ";

/**
 * @description <some description>
 *
 * @param  {Buffer} event 函数被调用时，传入的数据；
 *          特别地，当该函数被SyncStore-NoSQL的节点变化触发器触发调用时，
 *          假设：存储的数据对象为：{"a":{"b":"1"}}, 节点路径/a/b的值变成了2，
 *          那么，传入的数据如下：
 *          {
 *            "accessInfo": {
 *              "clientIp": "xx.xx.xx.xx", // 引起本次变更的客户端ip(如果是管理平台引起的变更，该值为"platform")
 *              "requestTimestamp": "1503555662573", // 引起本次变更的请求时间(服务端时间)
 *              "userToken": "xxx", // 本次变更操作的用户token，若未传递，该值为NULL_TOKEN（可透传给syncstore admin sdk进行后续操作）
 *              "deviceToken": "xxx" // 本次变更操作的设备token，若未传递，该值为NULL_TOKEN（可透传给syncstore admin sdk进行后续操作）
 *            },
 *            "data": {
 *              "path": "/a/b", // 节点路径
 *              "current": {  // 当前变化节点路径(path)对应的(变化后的)节点数据，若节点变化为删除，该处为空对象
 *                "b": "2"
 *              },
 *              "currentDataLength": x, // current的字节长度
 *              "currentDataId": "xxx", // current的数据ID
 *              "previous": {  // 当前变化节点路径(path)对应的(变化前的)节点数据，若节点变化为新增，该处为空对象
 *                "b": "1"
 *              },
 *              "preDataLength": x, // previous的字节长度
 *              "preDataId": "xxx", // previous的数据ID
 *            },
 *            "preFunction": {  // 当触发器关联的函数调用方式为串行，那么函数将可获取到前一个函数的调用返回等信息; 其它场景下，无该节点
 *              "funcName": "xxx", // 函数名称
 *              "fileName": "xxx.js", // 函数所属的文件(.js 或 .zip)
 *              "callback": { // 回调信息
 *                "requestId": "xxx", // 该函数被调用时的请求ID
 *                "result": { // 返回的结果，该节点可以是字符串或JSON对象，取决于函数返回的数据类型
 *                  "xx": "xxx"
 *                }
 *              }
 *            },
 *            "namespace": "xxx", // 数据存储空间唯一标识
 *            "src": "nosql",  // 触发器源名称
 *            "targetDevice": "xxx",  // 本次变更操作的设备token对应的设备ID
 *            "targetUser": "xxx",  // 本次变更操作的用户token对应的用户ID
 *            "projectId": "xxx",  // 当前namespace相应的project ID
 *          }
 *          注意：当preDataLength+currentDataLength > 5Mb，current和previous为 {}
 * @param  {object} context 是函数计算在运行时生成的一个对象，其中包含一些运行时的信息;
 * @param  {function} callback 函数结束的回调函数 (error, message)；
 *      Nodejs采用异步编程的模型，用户函数必须通过调用callback来返回数据或者错误。
 *          注意：确保callback被调用，否则，系统将认为函数没有结束，会等待函数结果直到超时；
 *          另外，callback被调用后则函数结束。
 */
module.exports.someFunc = function(event, context, callback) {
  // parse event
  let eventJson = {};
  let eventData = event.toString('utf8');

  try {
    console.log(TAG, "eventData=" + eventData);
    
    // eventJson = JSON.parse(eventData);
    // console.log(TAG, "event.src=" + eventJson.src);
    // console.log(TAG, "event.namespace=" + eventJson.namespace);
    // console.log(TAG, "event.targetDevice=" + eventJson.targetDevice);
    // console.log(TAG, "event.targetUser=" + eventJson.targetUser);
    // console.log(TAG, "event.data.path=" + eventJson.data.path);
    // console.log(TAG, "event.data.current=" + JSON.stringify(eventJson.data.current));
    // console.log(TAG, "event.data.previous=" + JSON.stringify(eventJson.data.previous));
    // console.log(TAG, "event.accessInfo.clientIp=" + eventJson.accessInfo.clientIp);
    // console.log(TAG, "event.accessInfo.requestTimestamp=" + eventJson.accessInfo.requestTimestamp);
    // console.log(TAG, "event.accessInfo.userToken=" + eventJson.accessInfo.userToken);
    // console.log(TAG, "event.accessInfo.deviceToken=" + eventJson.accessInfo.deviceToken);
  } catch(e) {
    console.error(TAG, e.message);
    callback(new Error("provided event parameter is wrong"), null);
    return;
  }

  // do something...
  callback(null, "succeed");
}
